@@ -4,7 +4,6 @@
 use std::{cmp::Reverse, env};
 
 use bumpalo::Bump;
-use itertools::Itertools;
 use radix_heap::RadixHeapMap;
 
 use crate::{
@@ -18,23 +17,23 @@ use crate::{
 pub(crate) struct ExceededGraphLimit {}
 
 /// Return the shortest route from `start` to the end vertex.
-fn shortest_vertex_path<'s, 'b>(
-    start: &'b Vertex<'s, 'b>,
-    vertex_arena: &'b Bump,
+fn shortest_vertex_path<'s, 'v>(
+    start: &'v Vertex<'s, 'v>,
+    vertex_arena: &'v Bump,
     size_hint: usize,
     graph_limit: usize,
-) -> Result<Vec<&'b Vertex<'s, 'b>>, ExceededGraphLimit> {
+) -> Result<Vec<&'v Vertex<'s, 'v>>, ExceededGraphLimit> {
     // We want to visit nodes with the shortest distance first, but
     // RadixHeapMap is a max-heap. Ensure nodes are wrapped with
     // Reverse to flip comparisons.
-    let mut heap: RadixHeapMap<Reverse<_>, &'b Vertex<'s, 'b>> = RadixHeapMap::new();
+    let mut heap: RadixHeapMap<Reverse<_>, &'v Vertex<'s, 'v>> = RadixHeapMap::new();
 
     heap.push(Reverse(0), start);
 
     let mut seen = DftHashMap::default();
     seen.reserve(size_hint);
 
-    let end: &'b Vertex<'s, 'b> = loop {
+    let end: &'v Vertex<'s, 'v> = loop {
         match heap.pop() {
             Some((Reverse(distance), current)) => {
                 if current.is_end() {
@@ -78,7 +77,7 @@ fn shortest_vertex_path<'s, 'b>(
     );
 
     let mut current = Some((0, end));
-    let mut vertex_route: Vec<&'b Vertex<'s, 'b>> = vec![];
+    let mut vertex_route: Vec<&'v Vertex<'s, 'v>> = vec![];
     while let Some((_, node)) = current {
         vertex_route.push(node);
         current = node.predecessor.get();
@@ -88,9 +87,9 @@ fn shortest_vertex_path<'s, 'b>(
     Ok(vertex_route)
 }
 
-fn shortest_path_with_edges<'s, 'b>(
-    route: &[&'b Vertex<'s, 'b>],
-) -> Vec<(Edge, &'b Vertex<'s, 'b>)> {
+fn shortest_path_with_edges<'s, 'v>(
+    route: &[&'v Vertex<'s, 'v>],
+) -> Vec<(Edge, &'v Vertex<'s, 'v>)> {
     let mut prev = route.first().expect("Expected non-empty route");
 
     let mut cost = 0;
@@ -112,18 +111,18 @@ fn shortest_path_with_edges<'s, 'b>(
 ///
 /// The vec returned does not return the very last vertex. This is
 /// necessary because a route of N vertices only has N-1 edges.
-fn shortest_path<'s, 'b>(
-    start: Vertex<'s, 'b>,
-    vertex_arena: &'b Bump,
+fn shortest_path<'s, 'v>(
+    start: Vertex<'s, 'v>,
+    vertex_arena: &'v Bump,
     size_hint: usize,
     graph_limit: usize,
-) -> Result<Vec<(Edge, &'b Vertex<'s, 'b>)>, ExceededGraphLimit> {
-    let start: &'b Vertex<'s, 'b> = vertex_arena.alloc(start);
+) -> Result<Vec<(Edge, &'v Vertex<'s, 'v>)>, ExceededGraphLimit> {
+    let start: &'v Vertex<'s, 'v> = vertex_arena.alloc(start);
     let vertex_path = shortest_vertex_path(start, vertex_arena, size_hint, graph_limit)?;
     Ok(shortest_path_with_edges(&vertex_path))
 }
 
-fn edge_between<'s, 'b>(before: &Vertex<'s, 'b>, after: &Vertex<'s, 'b>) -> Edge {
+fn edge_between<'s, 'v>(before: &Vertex<'s, 'v>, after: &Vertex<'s, 'v>) -> Edge {
     assert_ne!(before, after);
 
     let mut shortest_edge: Option<Edge> = None;
@@ -157,33 +156,20 @@ fn edge_between<'s, 'b>(before: &Vertex<'s, 'b>, after: &Vertex<'s, 'b>) -> Edge
 
 /// What is the total number of AST nodes?
 fn node_count(root: Option<&Syntax>) -> u32 {
-    let mut node = root;
-    let mut count = 0;
-    while let Some(current_node) = node {
-        let current_count = match current_node {
-            Syntax::List {
-                num_descendants, ..
-            } => *num_descendants,
-            Syntax::Atom { .. } => 1,
-        };
-        count += current_count;
+    let iter = std::iter::successors(root, |node| node.next_sibling());
 
-        node = current_node.next_sibling();
-    }
-
-    count
+    iter.map(|node| match node {
+        Syntax::List {
+            num_descendants, ..
+        } => *num_descendants,
+        Syntax::Atom { .. } => 1,
+    })
+    .sum::<u32>()
 }
 
 /// How many top-level AST nodes do we have?
 fn tree_count(root: Option<&Syntax>) -> u32 {
-    let mut node = root;
-    let mut count = 0;
-    while let Some(current_node) = node {
-        count += 1;
-        node = current_node.next_sibling();
-    }
-
-    count
+    std::iter::successors(root, |node| node.next_sibling()).count() as _
 }
 
 pub(crate) fn mark_syntax<'a>(
@@ -240,7 +226,7 @@ pub(crate) fn mark_syntax<'a>(
                 )
             })
             .take(print_length)
-            .collect_vec()
+            .collect::<Vec<_>>()
     );
 
     populate_change_map(&route, change_map);
@@ -249,7 +235,6 @@ pub(crate) fn mark_syntax<'a>(
 
 #[cfg(test)]
 mod tests {
-    use itertools::Itertools;
     use line_numbers::SingleLineSpan;
     use typed_arena::Arena;
 
@@ -273,16 +258,16 @@ mod tests {
     fn identical_atoms() {
         let arena = Arena::new();
 
-        let lhs = Syntax::new_atom(&arena, pos_helper(0), "foo", AtomKind::Normal);
+        let lhs = Syntax::new_atom(&arena, pos_helper(0), "foo".to_owned(), AtomKind::Normal);
         // Same content as LHS.
-        let rhs = Syntax::new_atom(&arena, pos_helper(0), "foo", AtomKind::Normal);
+        let rhs = Syntax::new_atom(&arena, pos_helper(0), "foo".to_owned(), AtomKind::Normal);
         init_all_info(&[lhs], &[rhs]);
 
         let start = Vertex::new(Some(lhs), Some(rhs));
         let vertex_arena = Bump::new();
         let route = shortest_path(start, &vertex_arena, 0, DEFAULT_GRAPH_LIMIT).unwrap();
 
-        let actions = route.iter().map(|(action, _)| *action).collect_vec();
+        let actions = route.iter().map(|(action, _)| *action).collect::<Vec<_>>();
         assert_eq!(
             actions,
             vec![UnchangedNode {
@@ -303,7 +288,7 @@ mod tests {
             vec![Syntax::new_atom(
                 &arena,
                 pos_helper(1),
-                "foo",
+                "foo".to_owned(),
                 AtomKind::Normal,
             )],
             "]",
@@ -324,7 +309,7 @@ mod tests {
         let vertex_arena = Bump::new();
         let route = shortest_path(start, &vertex_arena, 0, DEFAULT_GRAPH_LIMIT).unwrap();
 
-        let actions = route.iter().map(|(action, _)| *action).collect_vec();
+        let actions = route.iter().map(|(action, _)| *action).collect::<Vec<_>>();
         assert_eq!(
             actions,
             vec![
@@ -354,8 +339,8 @@ mod tests {
             "[",
             pos_helper(0),
             vec![
-                Syntax::new_atom(&arena, pos_helper(1), "foo", AtomKind::Normal),
-                Syntax::new_atom(&arena, pos_helper(2), "foo", AtomKind::Normal),
+                Syntax::new_atom(&arena, pos_helper(1), "foo".to_owned(), AtomKind::Normal),
+                Syntax::new_atom(&arena, pos_helper(2), "foo".to_owned(), AtomKind::Normal),
             ],
             "]",
             pos_helper(3),
@@ -366,7 +351,7 @@ mod tests {
         let vertex_arena = Bump::new();
         let route = shortest_path(start, &vertex_arena, 0, DEFAULT_GRAPH_LIMIT).unwrap();
 
-        let actions = route.iter().map(|(action, _)| *action).collect_vec();
+        let actions = route.iter().map(|(action, _)| *action).collect::<Vec<_>>();
         assert_eq!(
             actions,
             vec![
@@ -389,7 +374,7 @@ mod tests {
             pos_helper(0),
             vec![
                 Syntax::new_list(&arena, "(", pos_helper(1), vec![], ")", pos_helper(2)),
-                Syntax::new_atom(&arena, pos_helper(3), "foo", AtomKind::Normal),
+                Syntax::new_atom(&arena, pos_helper(3), "foo".to_owned(), AtomKind::Normal),
             ],
             "]",
             pos_helper(4),
@@ -401,7 +386,7 @@ mod tests {
             pos_helper(0),
             vec![
                 Syntax::new_list(&arena, "(", pos_helper(1), vec![], ")", pos_helper(2)),
-                Syntax::new_atom(&arena, pos_helper(3), "foo", AtomKind::Normal),
+                Syntax::new_atom(&arena, pos_helper(3), "foo".to_owned(), AtomKind::Normal),
             ],
             "}",
             pos_helper(4),
@@ -412,7 +397,7 @@ mod tests {
         let vertex_arena = Bump::new();
         let route = shortest_path(start, &vertex_arena, 0, DEFAULT_GRAPH_LIMIT).unwrap();
 
-        let actions = route.iter().map(|(action, _)| *action).collect_vec();
+        let actions = route.iter().map(|(action, _)| *action).collect::<Vec<_>>();
         assert_eq!(
             actions,
             vec![
@@ -437,14 +422,14 @@ mod tests {
         let lhs = vec![Syntax::new_atom(
             &arena,
             pos_helper(1),
-            "the quick brown fox",
+            "the quick brown fox".to_owned(),
             AtomKind::Comment,
         )];
 
         let rhs = vec![Syntax::new_atom(
             &arena,
             pos_helper(1),
-            "the quick brown cat",
+            "the quick brown cat".to_owned(),
             AtomKind::Comment,
         )];
         init_all_info(&lhs, &rhs);
@@ -453,7 +438,7 @@ mod tests {
         let vertex_arena = Bump::new();
         let route = shortest_path(start, &vertex_arena, 0, DEFAULT_GRAPH_LIMIT).unwrap();
 
-        let actions = route.iter().map(|(action, _)| *action).collect_vec();
+        let actions = route.iter().map(|(action, _)| *action).collect::<Vec<_>>();
         assert_eq!(
             actions,
             vec![ReplacedComment {
@@ -469,14 +454,14 @@ mod tests {
         let lhs = vec![Syntax::new_atom(
             &arena,
             pos_helper(1),
-            "the quick brown fox",
+            "the quick brown fox".to_owned(),
             AtomKind::Comment,
         )];
 
         let rhs = vec![Syntax::new_atom(
             &arena,
             pos_helper(1),
-            "foo bar",
+            "foo bar".to_owned(),
             AtomKind::Comment,
         )];
         init_all_info(&lhs, &rhs);
@@ -485,7 +470,7 @@ mod tests {
         let vertex_arena = Bump::new();
         let route = shortest_path(start, &vertex_arena, 0, DEFAULT_GRAPH_LIMIT).unwrap();
 
-        let actions = route.iter().map(|(action, _)| *action).collect_vec();
+        let actions = route.iter().map(|(action, _)| *action).collect::<Vec<_>>();
         assert_eq!(
             actions,
             vec![ReplacedComment {
@@ -502,13 +487,13 @@ mod tests {
             Syntax::new_atom(
                 &arena,
                 pos_helper(1),
-                "the quick brown fox",
+                "the quick brown fox".to_owned(),
                 AtomKind::Comment,
             ),
             Syntax::new_atom(
                 &arena,
                 pos_helper(2),
-                "the quick brown thing",
+                "the quick brown thing".to_owned(),
                 AtomKind::Comment,
             ),
         ];
@@ -516,7 +501,7 @@ mod tests {
         let rhs = vec![Syntax::new_atom(
             &arena,
             pos_helper(1),
-            "the quick brown fox.",
+            "the quick brown fox.".to_owned(),
             AtomKind::Comment,
         )];
         init_all_info(&lhs, &rhs);
@@ -525,7 +510,7 @@ mod tests {
         let vertex_arena = Bump::new();
         let route = shortest_path(start, &vertex_arena, 0, DEFAULT_GRAPH_LIMIT).unwrap();
 
-        let actions = route.iter().map(|(action, _)| *action).collect_vec();
+        let actions = route.iter().map(|(action, _)| *action).collect::<Vec<_>>();
         assert_eq!(
             actions,
             vec![
@@ -540,8 +525,8 @@ mod tests {
     #[test]
     fn mark_syntax_equal_atoms() {
         let arena = Arena::new();
-        let lhs = Syntax::new_atom(&arena, pos_helper(1), "foo", AtomKind::Normal);
-        let rhs = Syntax::new_atom(&arena, pos_helper(1), "foo", AtomKind::Normal);
+        let lhs = Syntax::new_atom(&arena, pos_helper(1), "foo".to_owned(), AtomKind::Normal);
+        let rhs = Syntax::new_atom(&arena, pos_helper(1), "foo".to_owned(), AtomKind::Normal);
         init_all_info(&[lhs], &[rhs]);
 
         let mut change_map = ChangeMap::default();
@@ -554,8 +539,8 @@ mod tests {
     #[test]
     fn mark_syntax_different_atoms() {
         let arena = Arena::new();
-        let lhs = Syntax::new_atom(&arena, pos_helper(1), "foo", AtomKind::Normal);
-        let rhs = Syntax::new_atom(&arena, pos_helper(1), "bar", AtomKind::Normal);
+        let lhs = Syntax::new_atom(&arena, pos_helper(1), "foo".to_owned(), AtomKind::Normal);
+        let rhs = Syntax::new_atom(&arena, pos_helper(1), "bar".to_owned(), AtomKind::Normal);
         init_all_info(&[lhs], &[rhs]);
 
         let mut change_map = ChangeMap::default();
