@@ -26,7 +26,7 @@ pub(crate) enum BackgroundColor {
 
 impl BackgroundColor {
     pub(crate) fn is_dark(self) -> bool {
-        matches!(self, BackgroundColor::Dark)
+        matches!(self, Self::Dark)
     }
 }
 
@@ -324,17 +324,59 @@ pub(crate) fn novel_style(style: Style, side: Side, background: BackgroundColor)
     }
 }
 
+/// Merge spans where the end of one span matches the start of the
+/// next span.
+///
+/// This reduces the number of ANSI character codes in the
+/// output. This is negligible for performance, but makes regression
+/// testing easier for difftastic.
+///
+/// The file compare.expected contains hashes of the output, so it
+/// considers `<green>ab</green>` to be distinct from
+/// `<green>a</green><green>b</green>`. Merging the spans normalises
+/// the output to `<green>ab</green>`.
+fn merge_adjacent(items: &[(SingleLineSpan, Style)]) -> Vec<(SingleLineSpan, Style)> {
+    let mut merged: Vec<(SingleLineSpan, Style)> = vec![];
+    let mut prev_item: Option<(SingleLineSpan, Style)> = None;
+
+    for (span, style) in items.iter().copied() {
+        match prev_item.take() {
+            Some((mut prev_span, prev_style)) => {
+                if prev_style == style
+                    && prev_span.line == span.line
+                    && prev_span.end_col == span.start_col
+                {
+                    prev_span.end_col = span.end_col;
+                    prev_item = Some((prev_span, style));
+                } else {
+                    merged.push((prev_span, prev_style));
+                    prev_item = Some((span, style));
+                }
+            }
+            None => {
+                prev_item = Some((span, style));
+            }
+        }
+    }
+
+    if let Some(last_item) = prev_item {
+        merged.push(last_item);
+    }
+
+    merged
+}
+
 pub(crate) fn color_positions(
     side: Side,
     background: BackgroundColor,
     syntax_highlight: bool,
     file_format: &FileFormat,
-    positions: &[MatchedPos],
+    mps: &[MatchedPos],
 ) -> Vec<(SingleLineSpan, Style)> {
     let mut styles = vec![];
-    for pos in positions {
+    for mp in mps {
         let mut style = Style::new();
-        match pos.kind {
+        match mp.kind {
             MatchKind::UnchangedToken { highlight, .. } | MatchKind::Ignored { highlight } => {
                 if syntax_highlight {
                     if let TokenKind::Atom(atom_kind) = highlight {
@@ -400,9 +442,10 @@ pub(crate) fn color_positions(
                 }
             }
         };
-        styles.push((pos.pos, style));
+        styles.push((mp.pos, style));
     }
-    styles
+
+    merge_adjacent(&styles)
 }
 
 pub(crate) fn apply_colors(
@@ -411,9 +454,9 @@ pub(crate) fn apply_colors(
     syntax_highlight: bool,
     file_format: &FileFormat,
     background: BackgroundColor,
-    positions: &[MatchedPos],
+    mps: &[MatchedPos],
 ) -> Vec<String> {
-    let styles = color_positions(side, background, syntax_highlight, file_format, positions);
+    let styles = color_positions(side, background, syntax_highlight, file_format, mps);
     let lines = split_on_newlines(s).collect::<Vec<_>>();
     style_lines(&lines, &styles)
 }
@@ -457,6 +500,19 @@ pub(crate) fn print_warning(s: &str, display_options: &DisplayOptions) {
     eprint!("{}\n\n", s);
 }
 
+/// Style `s` as an error and write to stderr.
+pub(crate) fn print_error(s: &str, use_color: bool) {
+    // TODO: this is inconsistent with print_warning regarding
+    // arguments and trailing whitespace.
+    let prefix = if use_color {
+        "error: ".red().bold().to_string()
+    } else {
+        "error: ".to_owned()
+    };
+
+    eprintln!("{}{}", prefix, s);
+}
+
 pub(crate) fn apply_line_number_color(
     s: &str,
     is_novel: bool,
