@@ -4,6 +4,9 @@
 //! manual](http://difftastic.wilfred.me.uk/).
 //!
 
+// I frequently develop difftastic on a newer rustc than the MSRV, so
+// these two aren't relevant.
+#![allow(renamed_and_removed_lints)]
 // This tends to trigger on larger tuples of simple types, and naming
 // them would probably be worse for readability.
 #![allow(clippy::type_complexity)]
@@ -19,9 +22,17 @@
 // implementation does not consider the mutable fields, so it is still
 // correct.
 #![allow(clippy::mutable_key_type)]
+// manual_unwrap_or_default was added in Rust 1.79, so earlier versions of
+// clippy complain about allowing it.
+#![allow(unknown_lints)]
 // It's sometimes more readable to explicitly create a vec than to use
 // the Default trait.
 #![allow(clippy::manual_unwrap_or_default)]
+// I find the explicit arithmetic clearer sometimes.
+#![allow(clippy::implicit_saturating_sub)]
+// It's helpful being super explicit about byte length versus Unicode
+// character point length sometimes.
+#![allow(clippy::needless_as_bytes)]
 // .to_owned() is more explicit on string references.
 #![warn(clippy::str_to_string)]
 // .to_string() on a String is clearer as .clone().
@@ -60,6 +71,7 @@ use crate::diff::dijkstra::ExceededGraphLimit;
 use crate::diff::{dijkstra, unchanged};
 use crate::display::context::opposite_positions;
 use crate::display::hunks::{matched_pos_to_hunks, merge_adjacent};
+use crate::display::style::print_error;
 use crate::exit_codes::EXIT_BAD_ARGUMENTS;
 use crate::exit_codes::{EXIT_FOUND_CHANGES, EXIT_SUCCESS};
 use crate::files::{
@@ -86,10 +98,10 @@ use crate::parse::syntax;
 ///
 /// For reference, Jemalloc uses 10-20% more time (although up to 33%
 /// more instructions) when testing on sample files.
-#[cfg(not(target_env = "msvc"))]
+#[cfg(not(any(target_env = "msvc", target_os = "illumos", target_os = "freebsd")))]
 use tikv_jemallocator::Jemalloc;
 
-#[cfg(not(target_env = "msvc"))]
+#[cfg(not(any(target_env = "msvc", target_os = "illumos", target_os = "freebsd")))]
 #[global_allocator]
 static GLOBAL: Jemalloc = Jemalloc;
 
@@ -239,6 +251,7 @@ fn main() {
             display_options,
             set_exit_code,
             language_overrides,
+            binary_overrides,
         } => {
             let diff_result = diff_conflicts_file(
                 &display_path,
@@ -246,6 +259,7 @@ fn main() {
                 &display_options,
                 &diff_options,
                 &language_overrides,
+                &binary_overrides,
             );
 
             print_diff_result(&display_options, &diff_result);
@@ -262,6 +276,7 @@ fn main() {
             display_options,
             set_exit_code,
             language_overrides,
+            binary_overrides,
             lhs_path,
             rhs_path,
             lhs_permissions,
@@ -297,6 +312,7 @@ fn main() {
                         &display_options,
                         &diff_options,
                         &language_overrides,
+                        &binary_overrides,
                     );
 
                     if matches!(display_options.display_mode, DisplayMode::Json) {
@@ -351,6 +367,7 @@ fn main() {
                         &diff_options,
                         false,
                         &language_overrides,
+                        &binary_overrides,
                     );
                     if diff_result.has_reportable_change() {
                         encountered_changes = true;
@@ -389,10 +406,20 @@ fn diff_file(
     diff_options: &DiffOptions,
     missing_as_empty: bool,
     overrides: &[(LanguageOverride, Vec<glob::Pattern>)],
+    binary_overrides: &[glob::Pattern],
 ) -> DiffResult {
     let (lhs_bytes, rhs_bytes) = read_files_or_die(lhs_path, rhs_path, missing_as_empty);
-    let (mut lhs_src, mut rhs_src) = match (guess_content(&lhs_bytes), guess_content(&rhs_bytes)) {
+
+    let (mut lhs_src, mut rhs_src) = match (
+        guess_content(&lhs_bytes, lhs_path, binary_overrides),
+        guess_content(&rhs_bytes, rhs_path, binary_overrides),
+    ) {
         (ProbableFileKind::Binary, _) | (_, ProbableFileKind::Binary) => {
+            let has_byte_changes = if lhs_bytes == rhs_bytes {
+                None
+            } else {
+                Some((lhs_bytes.len(), rhs_bytes.len()))
+            };
             return DiffResult {
                 extra_info: renamed,
                 display_path: display_path.to_owned(),
@@ -402,7 +429,7 @@ fn diff_file(
                 lhs_positions: vec![],
                 rhs_positions: vec![],
                 hunks: vec![],
-                has_byte_changes: lhs_bytes != rhs_bytes,
+                has_byte_changes,
                 has_syntactic_changes: false,
             };
         }
@@ -467,12 +494,16 @@ fn diff_conflicts_file(
     display_options: &DisplayOptions,
     diff_options: &DiffOptions,
     overrides: &[(LanguageOverride, Vec<glob::Pattern>)],
+    binary_overrides: &[glob::Pattern],
 ) -> DiffResult {
     let bytes = read_file_or_die(path);
-    let mut src = match guess_content(&bytes) {
+    let mut src = match guess_content(&bytes, path, binary_overrides) {
         ProbableFileKind::Text(src) => src,
         ProbableFileKind::Binary => {
-            eprintln!("error: Expected a text file with conflict markers, got a binary file.");
+            print_error(
+                "Expected a text file with conflict markers, got a binary file.",
+                display_options.use_color,
+            );
             std::process::exit(EXIT_BAD_ARGUMENTS);
         }
     };
@@ -484,15 +515,22 @@ fn diff_conflicts_file(
     let conflict_files = match apply_conflict_markers(&src) {
         Ok(cf) => cf,
         Err(msg) => {
-            eprintln!("error: {}", msg);
+            print_error(&msg, display_options.use_color);
             std::process::exit(EXIT_BAD_ARGUMENTS);
         }
     };
 
     if conflict_files.num_conflicts == 0 {
-        eprintln!(
-            "error: Difftastic requires two paths, or a single file with conflict markers {}.\n",
-            START_LHS_MARKER,
+        print_error(
+            &format!(
+                "Difftastic requires two paths, or a single file with conflict markers {}.\n",
+                if display_options.use_color {
+                    START_LHS_MARKER.bold().to_string()
+                } else {
+                    START_LHS_MARKER.to_owned()
+                }
+            ),
+            display_options.use_color,
         );
 
         eprintln!("USAGE:\n\n    {}\n", USAGE);
@@ -534,7 +572,11 @@ fn check_only_text(
     lhs_src: &str,
     rhs_src: &str,
 ) -> DiffResult {
-    let has_changes = lhs_src != rhs_src;
+    let has_byte_changes = if lhs_src == rhs_src {
+        None
+    } else {
+        Some((lhs_src.as_bytes().len(), rhs_src.as_bytes().len()))
+    };
 
     DiffResult {
         display_path: display_path.to_owned(),
@@ -545,8 +587,8 @@ fn check_only_text(
         lhs_positions: vec![],
         rhs_positions: vec![],
         hunks: vec![],
-        has_byte_changes: has_changes,
-        has_syntactic_changes: has_changes,
+        has_byte_changes,
+        has_syntactic_changes: lhs_src != rhs_src,
     }
 }
 
@@ -586,7 +628,7 @@ fn diff_file_content(
             lhs_positions: vec![],
             rhs_positions: vec![],
             hunks: vec![],
-            has_byte_changes: false,
+            has_byte_changes: None,
             has_syntactic_changes: false,
         };
     }
@@ -618,6 +660,13 @@ fn diff_file_content(
                         Ok((lhs, rhs)) => {
                             if diff_options.check_only {
                                 let has_syntactic_changes = lhs != rhs;
+
+                                let has_byte_changes = if lhs_src == rhs_src {
+                                    None
+                                } else {
+                                    Some((lhs_src.as_bytes().len(), rhs_src.as_bytes().len()))
+                                };
+
                                 return DiffResult {
                                     extra_info,
                                     display_path: display_path.to_owned(),
@@ -627,7 +676,7 @@ fn diff_file_content(
                                     lhs_positions: vec![],
                                     rhs_positions: vec![],
                                     hunks: vec![],
-                                    has_byte_changes: true,
+                                    has_byte_changes,
                                     has_syntactic_changes,
                                 };
                             }
@@ -760,6 +809,12 @@ fn diff_file_content(
     );
     let has_syntactic_changes = !hunks.is_empty();
 
+    let has_byte_changes = if lhs_src == rhs_src {
+        None
+    } else {
+        Some((lhs_src.as_bytes().len(), rhs_src.as_bytes().len()))
+    };
+
     DiffResult {
         extra_info,
         display_path: display_path.to_owned(),
@@ -769,7 +824,7 @@ fn diff_file_content(
         lhs_positions,
         rhs_positions,
         hunks,
-        has_byte_changes: true,
+        has_byte_changes,
         has_syntactic_changes,
     }
 }
@@ -786,10 +841,12 @@ fn diff_directories<'a>(
     display_options: &DisplayOptions,
     diff_options: &DiffOptions,
     overrides: &[(LanguageOverride, Vec<glob::Pattern>)],
+    binary_overrides: &[glob::Pattern],
 ) -> impl ParallelIterator<Item = DiffResult> + 'a {
     let diff_options = diff_options.clone();
     let display_options = display_options.clone();
     let overrides: Vec<_> = overrides.into();
+    let binary_overrides: Vec<_> = binary_overrides.into();
 
     // We greedily list all files in the directory, and then diff them
     // in parallel. This is assuming that diffing is slower than
@@ -813,6 +870,7 @@ fn diff_directories<'a>(
             &diff_options,
             true,
             &overrides,
+            &binary_overrides,
         )
     })
 }
@@ -905,7 +963,7 @@ fn print_diff_result(display_options: &DisplayOptions, summary: &DiffResult) {
             }
         }
         (FileContent::Binary, FileContent::Binary) => {
-            if display_options.print_unchanged || summary.has_byte_changes {
+            if display_options.print_unchanged || summary.has_byte_changes.is_some() {
                 println!(
                     "{}",
                     display::style::header(
@@ -917,10 +975,37 @@ fn print_diff_result(display_options: &DisplayOptions, summary: &DiffResult) {
                         display_options
                     )
                 );
-                if summary.has_byte_changes {
-                    println!("Binary contents changed.\n");
-                } else {
-                    println!("No changes.\n");
+
+                match summary.has_byte_changes {
+                    Some((lhs_len, rhs_len)) => {
+                        let format_options = FormatSizeOptions::from(BINARY).decimal_places(1);
+
+                        if lhs_len == 0 {
+                            // Strictly speaking this is wrong:
+                            // previously we may have had an empty but
+                            // existent file. In that case, it's a
+                            // file modification instead of a file
+                            // creation.
+                            //
+                            // TODO: Fix this pedantic case.
+                            println!(
+                                "Binary file added ({}).\n",
+                                &format_size(rhs_len, format_options),
+                            )
+                        } else if rhs_len == 0 {
+                            println!(
+                                "Binary file removed ({}).\n",
+                                &format_size(lhs_len, format_options),
+                            )
+                        } else {
+                            println!(
+                                "Binary file modified (old: {}, new: {}).\n",
+                                &format_size(lhs_len, format_options),
+                                &format_size(rhs_len, format_options),
+                            )
+                        }
+                    }
+                    None => println!("No changes.\n"),
                 }
             }
         }
