@@ -9,9 +9,10 @@ use std::{
 
 use clap::{crate_authors, crate_description, value_parser, Arg, ArgAction, Command};
 use crossterm::tty::IsTty;
+use owo_colors::OwoColorize as _;
 
 use crate::{
-    display::style::BackgroundColor,
+    display::style::{print_error, BackgroundColor},
     exit_codes::EXIT_BAD_ARGUMENTS,
     parse::guess_language::{language_override_from_name, LanguageOverride},
     version::VERSION,
@@ -90,28 +91,64 @@ impl Default for DiffOptions {
 }
 
 fn app() -> clap::Command {
+    let bin_name = env!("CARGO_BIN_NAME");
+
+    let mut after_help = String::new();
+    after_help
+        .push_str("You can compare two files with difftastic by specifying them as arguments.\n\n");
+    after_help.push_str(&format!("$ {} old.js new.js", bin_name).bold().to_string());
+
+    after_help.push_str("\n\nYou can also use directories as arguments. Difftastic will walk both directories and compare files with matching names.\n\n");
+    after_help.push_str(&format!("$ {} old/ new/", bin_name).bold().to_string());
+
+    after_help.push_str("\n\nIf you have a file with conflict markers, you can pass it as a single argument. Difftastic will diff the two conflicting file states.\n\n");
+    after_help.push_str(
+        &format!("$ {} file_with_conflicts.js", bin_name)
+            .bold()
+            .to_string(),
+    );
+
+    // For some reason clap will hard wrap these invocations weirdly
+    // (with extra blank lines) if we use bold. Since these are
+    // showing CLI formats rather than concrete values, compromise by
+    // not using bold.
+    after_help.push_str("\n\nDifftastic can also be invoked with 7 or 9 arguments in the format that GIT_EXTERNAL_DIFF expects.\n\n");
+    after_help.push_str(&format!(
+        "$ {} DISPLAY-PATH OLD-FILE OLD-HEX OLD-MODE NEW-FILE NEW-HEX NEW-MODE",
+        bin_name
+    ));
+
+    after_help.push('\n');
+    after_help.push_str(&format!(
+        "$ {} OLD-NAME OLD-FILE OLD-HEX OLD-MODE NEW-FILE NEW-HEX NEW-MODE NEW-NAME METADATA",
+        bin_name
+    ));
+
+    after_help.push_str("\n\nSee the full manual at ");
+    if std::io::stdout().is_tty() {
+        // Make the link to the manual clickable in terminals that
+        // support OSC 8, the ANSI escape code for hyperlinks.
+        //
+        // https://gist.github.com/egmontkob/eb114294efbcd5adb1944c9f3cb5feda
+        // https://github.com/Alhadis/OSC8-Adoption
+        //
+        // There isn't any way of detecting whether the terminal
+        // supports OSC 8 specifically, but we can limit usage to when
+        // there's a TTY. This is similar to how we detect whether to
+        // use colour.
+        after_help.push_str("\x1b]8;;https://difftastic.wilfred.me.uk/\x1b\\https://difftastic.wilfred.me.uk/\x1b]8;;\x1b\\");
+    } else {
+        after_help.push_str("https://difftastic.wilfred.me.uk/");
+    }
+    after_help.push('.');
+
     Command::new("Difftastic")
         .override_usage(USAGE)
         .version(env!("CARGO_PKG_VERSION"))
         .long_version(VERSION.as_str())
         .about(crate_description!())
         .author(crate_authors!())
-        .after_long_help(concat!(
-            "You can compare two files with difftastic by specifying them as arguments.\n\n",
-            "$ ",
-            env!("CARGO_BIN_NAME"),
-            " old.js new.js\n\n",
-            "You can also use directories as arguments. Difftastic will walk both directories and compare files with matching names.\n\n",
-            "$ ",
-            env!("CARGO_BIN_NAME"),
-            " old/ new/\n\n",
-            "If you have a file with conflict markers, you can pass it as a single argument. Difftastic will diff the two conflicting file states.\n\n",
-            "$ ",
-            env!("CARGO_BIN_NAME"),
-            " file_with_conflicts.js\n\n",
-            "Difftastic can also be invoked with 7 arguments in the format that GIT_EXTERNAL_DIFF expects.\n\n",
-            "See the full manual at: https://difftastic.wilfred.me.uk/")
-        )
+        .after_long_help(after_help)
         .arg(
             Arg::new("dump-syntax")
                 .long("dump-syntax")
@@ -271,6 +308,25 @@ $ export DFT_OVERRIDE_2='*.js:javascript jsx'
 When multiple overrides are specified, the first matching override wins."))
                 .env("DFT_OVERRIDE")
         )
+        .arg(
+            Arg::new("override-binary").long("override-binary")
+                .value_name("GLOB")
+                .action(ArgAction::Append)
+                .help(concat!("Treat file names matching this glob as binary files, overriding normal binary detection. For example:
+
+$ ", env!("CARGO_BIN_NAME"), " --override-binary='*.gz' old.gz new.gz
+
+This argument may be given more than once. For example:
+
+$ ", env!("CARGO_BIN_NAME"), " --override-binary='*.gz' --override-binary='foo.pickle' old.gz new.gz
+
+To configure multiple overrides using environment variables, difftastic also accepts DFT_OVERRIDE_BINARY_1 up to DFT_OVERRIDE_BINARY_9.
+
+$ export DFT_OVERRIDE_BINARY='*.gz'
+$ export DFT_OVERRIDE_BINARY_1='*.bz2'
+$ export DFT_OVERRIDE_BINARY_2='foo.pickle'"))
+                .env("DFT_OVERRIDE_BINARY")
+        )
         .arg(
             Arg::new("list-languages").long("list-languages")
                 .action(ArgAction::SetTrue)
@@ -342,7 +398,7 @@ pub(crate) enum FileArgument {
 impl FileArgument {
     pub(crate) fn permissions(&self) -> Option<FilePermissions> {
         match self {
-            FileArgument::NamedPath(path) => {
+            Self::NamedPath(path) => {
                 // When used with `git difftool`, the first argument
                 // is a temporary file that always has the same
                 // permissions. That doesn't mean the file permissions
@@ -354,8 +410,8 @@ impl FileArgument {
                 let metadata = std::fs::metadata(path).ok()?;
                 Some(metadata.permissions().into())
             }
-            FileArgument::Stdin => None,
-            FileArgument::DevNull => None,
+            Self::Stdin => None,
+            Self::DevNull => None,
         }
     }
 }
@@ -428,11 +484,11 @@ impl FileArgument {
     /// argument.
     pub(crate) fn from_cli_argument(arg: &OsStr) -> Self {
         if arg == "/dev/null" {
-            FileArgument::DevNull
+            Self::DevNull
         } else if arg == "-" {
-            FileArgument::Stdin
+            Self::Stdin
         } else {
-            FileArgument::NamedPath(PathBuf::from(arg))
+            Self::NamedPath(PathBuf::from(arg))
         }
     }
 
@@ -441,9 +497,9 @@ impl FileArgument {
     pub(crate) fn from_path_argument(arg: &OsStr) -> Self {
         // For new and deleted files, Git passes `/dev/null` as the reference file.
         if arg == "/dev/null" {
-            FileArgument::DevNull
+            Self::DevNull
         } else {
-            FileArgument::NamedPath(PathBuf::from(arg))
+            Self::NamedPath(PathBuf::from(arg))
         }
     }
 }
@@ -451,11 +507,11 @@ impl FileArgument {
 impl Display for FileArgument {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         match self {
-            FileArgument::NamedPath(path) => {
+            Self::NamedPath(path) => {
                 write!(f, "{}", relative_to_current(path).display())
             }
-            FileArgument::Stdin => write!(f, "(stdin)"),
-            FileArgument::DevNull => write!(f, "/dev/null"),
+            Self::Stdin => write!(f, "(stdin)"),
+            Self::DevNull => write!(f, "/dev/null"),
         }
     }
 }
@@ -466,6 +522,7 @@ pub(crate) enum Mode {
         display_options: DisplayOptions,
         set_exit_code: bool,
         language_overrides: Vec<(LanguageOverride, Vec<glob::Pattern>)>,
+        binary_overrides: Vec<glob::Pattern>,
         /// The path where we can read the LHS file. This is often a
         /// temporary file generated by source control.
         lhs_path: FileArgument,
@@ -484,6 +541,7 @@ pub(crate) enum Mode {
         display_options: DisplayOptions,
         set_exit_code: bool,
         language_overrides: Vec<(LanguageOverride, Vec<glob::Pattern>)>,
+        binary_overrides: Vec<glob::Pattern>,
         path: FileArgument,
         /// The path that we show to the user.
         display_path: String,
@@ -629,6 +687,30 @@ fn parse_overrides_or_die(raw_overrides: &[String]) -> Vec<(LanguageOverride, Ve
     combined_overrides
 }
 
+fn parse_binary_overrides_or_die(glob_strs: &[String]) -> Vec<glob::Pattern> {
+    let mut overrides: Vec<glob::Pattern> = vec![];
+    let mut invalid_syntax = false;
+
+    for glob_str in glob_strs {
+        match glob::Pattern::new(glob_str) {
+            Ok(pattern) => {
+                overrides.push(pattern);
+            }
+            Err(e) => {
+                eprintln!("Invalid glob syntax '{}'", glob_str);
+                eprintln!("Glob parsing error: {}", e.msg);
+                invalid_syntax = true;
+            }
+        }
+    }
+
+    if invalid_syntax {
+        std::process::exit(EXIT_BAD_ARGUMENTS);
+    }
+
+    overrides
+}
+
 /// Parse CLI arguments passed to the binary.
 pub(crate) fn parse_args() -> Mode {
     let matches = app().get_matches();
@@ -649,6 +731,18 @@ pub(crate) fn parse_args() -> Mode {
 
     let ignore_comments = matches.get_flag("ignore-comments");
 
+    let mut raw_binary_overrides: Vec<String> = vec![];
+    if let Some(binary_overrides) = matches.get_many("override-binary") {
+        raw_binary_overrides = binary_overrides.cloned().collect();
+    }
+    for i in 1..=9 {
+        if let Ok(value) = env::var(format!("DFT_OVERRIDE_BINARY_{}", i)) {
+            raw_binary_overrides.push(value);
+        }
+    }
+
+    let binary_overrides = parse_binary_overrides_or_die(&raw_binary_overrides);
+
     let mut raw_overrides: Vec<String> = vec![];
     if let Some(overrides) = matches.get_many("override") {
         raw_overrides = overrides.cloned().collect();
@@ -786,7 +880,6 @@ pub(crate) fn parse_args() -> Mode {
         }
     }
 
-    // TODO: document these different ways of calling difftastic.
     let (display_path, lhs_path, rhs_path, lhs_permissions, rhs_permissions, renamed) = match &args
         [..]
     {
@@ -859,14 +952,18 @@ pub(crate) fn parse_args() -> Mode {
                 display_options,
                 set_exit_code,
                 language_overrides,
+                binary_overrides,
             };
         }
         _ => {
             if !args.is_empty() {
-                eprintln!(
-                    "error: Difftastic does not support being called with {} argument{}.\n",
-                    args.len(),
-                    if args.len() == 1 { "" } else { "s" }
+                print_error(
+                    &format!(
+                        "Difftastic does not support being called with {} argument{}.\n",
+                        args.len(),
+                        if args.len() == 1 { "" } else { "s" }
+                    ),
+                    use_color,
                 );
             }
             eprintln!("USAGE:\n\n    {}\n", USAGE);
@@ -892,6 +989,7 @@ pub(crate) fn parse_args() -> Mode {
         display_options,
         set_exit_code,
         language_overrides,
+        binary_overrides,
         lhs_path,
         rhs_path,
         lhs_permissions,
@@ -929,16 +1027,18 @@ fn detect_terminal_width() -> usize {
 pub(crate) fn should_use_color(color_output: ColorOutput) -> bool {
     match color_output {
         ColorOutput::Always => true,
-        ColorOutput::Auto => {
-            // Always enable colour if stdout is a TTY or if the git pager is active.
-            // TODO: consider following the env parsing logic in git_config_bool
-            // in config.c.
-            std::io::stdout().is_tty() || env::var("GIT_PAGER_IN_USE").is_ok()
-        }
+        ColorOutput::Auto => detect_color_support(),
         ColorOutput::Never => false,
     }
 }
 
+/// Always enable colour if stdout is a TTY or if the git pager is active.
+fn detect_color_support() -> bool {
+    // TODO: consider following the env parsing logic in git_config_bool
+    // in config.c.
+    std::io::stdout().is_tty() || env::var("GIT_PAGER_IN_USE").is_ok()
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
